#include <avr/pgmspace.h>
#include "include/config.h"
#include "include/font.h"
#include "include/t6963c.h"

/** *************************************************************
	@brief	Print a String from flash
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	struct1 : Struct with the font
	@param 	invers : 0=Normal , 1= Invers
	@return	none
*****************************************************************/
void GLCD_PRINT_p(unsigned char x,unsigned char y, const char *in, const struct FONT_DEF *struct1, unsigned char invers)
{
	unsigned int offset;
	unsigned char width, by=0, mask=0;
	unsigned char NrBytes;
	unsigned char i,j,map,ertefah,allwidth=0;

	while((map = pgm_read_byte(in++))) 
	{
		map = pgm_read_byte(&struct1->mapping_table[map]);

		width = struct1->glyph_width;
		if(width == 0)
			width = pgm_read_byte(&struct1->width_table[map]);

		offset = pgm_read_word(&struct1->offset_table[map]);
		ertefah = struct1->glyph_height;

		NrBytes = ((width-1)/8)+1;

		for(j=0; j < ertefah * NrBytes; j+=NrBytes )
		{   // ertefah
			for(i=0 ; i<width; i++)
			{   //  width
			    if(i%8 == 0)
			    {
			     by = pgm_read_byte(&struct1->glyph_table[ offset+j+(i/8) ]);
			     mask = 0x80;
			    }

				if( by & mask )
					GLCD_PIXEL(  x+i+allwidth , y+j/ NrBytes, !invers  );
	 			else
	 				GLCD_PIXEL(  x+i+allwidth , y+j/ NrBytes, invers  );
	 				
	 			mask >>= 1;
			}//End i
		}// End j
		allwidth+=width;
	}// End K
}

/** *************************************************************
	@brief	Print a String
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	struct1 : Struct with the font
	@param 	invers : 0=Normal , 1= Invers
	@return	none
*****************************************************************/
void GLCD_PRINT(unsigned char x,unsigned char y, const char *in, const struct FONT_DEF *struct1, unsigned char invers)
{
	unsigned int offset;
	unsigned char width, by=0, mask=0;
	unsigned char NrBytes;
	unsigned char i,j,map,ertefah,allwidth=0;

	while((map = *in++)) 
	{
		map = pgm_read_byte(&struct1->mapping_table[map]);

		width = struct1->glyph_width;
		if(width == 0)
			width = pgm_read_byte(&struct1->width_table[map]);

		offset = pgm_read_word(&struct1->offset_table[map]);
		ertefah = struct1->glyph_height;

		NrBytes = ((width-1)/8)+1;
	
		for(j=0; j<ertefah * NrBytes; j+=NrBytes    )
		{   // ertefah
			for(i=0 ; i<width  ; i++)
			{   //  width
			    if(i%8 == 0)
			    {
			     by = pgm_read_byte(&struct1->glyph_table[ offset+j+(i/8) ]);
			     mask = 0x80;
			    }

				if( by & mask )
					GLCD_PIXEL(  x+i+allwidth , y+j/ NrBytes, !invers  );
	 			else
	 				GLCD_PIXEL(  x+i+allwidth , y+j/ NrBytes, invers  );

	 			mask >>= 1;
			}//End i
		}// End j
		allwidth+=width;
	}// End K
}

/** *************************************************************
	@brief	Get lenght in pixels for a string
	@param 	in 	: The String
	@param 	struct1 : Struct with the font
	@return	none
*****************************************************************/
int GLCD_STRLEN(const char *in, const struct FONT_DEF *struct1)
{
	unsigned char width;
	unsigned char map,allwidth=0;

	while((map = *in++)) 
	{
		map = pgm_read_byte(&struct1->mapping_table[map]);

		width = struct1->glyph_width;
		if(width == 0)
			width = pgm_read_byte(&struct1->width_table[map]);

		allwidth+=width;
	}// End K
	return allwidth;
}

/** *************************************************************
	@brief	Print a character
	@param 	x 	: column
	@param 	y 	: row
	@param 	in 	: The String
	@param 	struct1 : Struct with the font
	@return	none
*****************************************************************/
uint8_t GLCD_CHAR(unsigned char x,unsigned char y, char in, const struct FONT_DEF *struct1)
{
  uint8_t map;
  uint8_t width;
  uint16_t offset;
  uint8_t height;
  unsigned char buffer[GLCD_BYTES_PER_ROW];
  
  uint8_t spalte,zeile;
  uint8_t counter=0;
  uint8_t buffer_counter=0;
  uint8_t pixel=0,bc;
  uint8_t o;
  
  map = in;									// Index des Zeichens
  map = pgm_read_byte(&struct1->mapping_table[map]);
  width = pgm_read_byte(&struct1->width_table[map]);				// Breite des Zeichens
  offset = pgm_read_word(&struct1->offset_table[map]);				// Die Startadresse, wo das Zeichen anfaengt
  height = struct1->glyph_height;						// Die Hoehe des Fonts
  
  for(zeile=0;zeile<=height;zeile++)						// Zeilenweise lesen wir das Zeichen
  {
    for(spalte=0;spalte<((width-1)/8)+1;spalte++)				// Wir lesen pro Spalte nur soviel bytes, wie benoetigt
    {
      buffer[buffer_counter]=pgm_read_byte(&struct1->glyph_table[offset+counter]);// Ein Byte lesen

      counter++;								// Zaehler erhoehen
      buffer_counter++;
      buffer[buffer_counter]=0;							// das naechste buffer byte loeschen, sonst steht muell drin
    }
    
    for(pixel=0;pixel<x-((x/8)*8);pixel++)					// Um soviel bits muessen wir das Zeichen nach rechts verschieben
    {
      for(bc=buffer_counter;bc>0;bc--)						// Rueckwaerts zaehlen, bis die Anzahl der gelesenen Bytes um 1 bit verschoben ist
      {
	buffer[bc]>>=1;								// Buffer um 1 bit nach rechts shiften
	buffer[bc]|=(buffer[bc-1] & 1<<0)<<7;					// Ueberlauf bit vom vorherigen byte holen
      }
      buffer[0]>>=1;								// Buffer 0 muss nur geshiftet werden, hier gibt's keinen Ueberlauf
    }

    GLCD_GOTO(x,y+zeile);							// Zur Position auf dem Screen gehen
    for(o=0;o<=buffer_counter;o++)						// Alle bytes + 1 durchzaehlen
    {
     
      GLCD_CPUT(0xC5);								// 0xC5 = Lese Display ohne adresse zu aendern
      buffer[o]|=GLCD_DGET();							// Hintergrund-byte mit buffer-byte zusammenfuegen
      GLCD_DPUT(buffer[o]);							// Den buffer ausgeben
      GLCD_CPUT(0xC0);								// 0xC0 = Schreibe und erhoehe die adresse +1
    }

    buffer_counter=0;

  }
  
  return width;									// Rueckgabewert ist die Breite des Zeichens
}

/** *************************************************************
	@brief	Get the hight of a font
	@param none
	@return	The height
*****************************************************************/
int GLCD_GET_FONT_HEIGHT( const struct FONT_DEF *struct1)
{
  return struct1->glyph_height;
}
