#include <util/delay.h>
#include "include/config.h"
#include "include/general.h"

/** *************************************************************
	@brief	Converts a HEX String to an integer
	@param 	in 	: The string
	@return	the integer
*****************************************************************/
int hextoint(char *in){
  int value=0;
  unsigned char i=0;
  while(1){
    if(in[i] >= '0' && in[i] <= '9'){
      value <<= 4;
      value |= in[i] - '0';
    }
    else if(in[i] >= 'A' && in[i] <= 'F'){
      value <<= 4;
      value |= in[i] - 'A'+10;
    }
      else if(in[i] >= 'a' && in[i] <= 'f'){
      value <<= 4;
      value |= in[i] - 'a'+10;
    }
    else
      break;
    i++;
  }
  return value;
}

/** *************************************************************
	@brief	Gives lenght of a string
	@param 	string : The string
	@return	the lenght
*****************************************************************/
uint8_t strlenght(const char *string) {
	const char *s;

	s = string;
	while (*s)
		s++;
	return s - string;
}

/** *************************************************************
	@brief	Reverse a string
	@param 	str : The original string
	@return	the reversed string
*****************************************************************/
char *strrev(char *str) {
	char *p1, *p2;

	if (!str || !*str)
		return str;

	for (p1 = str, p2 = str + strlenght(str) - 1; p2 > p1; ++p1, --p2) {
		*p1 ^= *p2;
		*p2 ^= *p1;
		*p1 ^= *p2;
	}

	return str;
}

/** *************************************************************
	@brief	Converts an integer into a string
	@param 	n :
	@param 	s :
	@param 	b :
	@return	the string
*****************************************************************/
char *inttostr(int n, char *s, int b) {
	static char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
	int i=0, sign;
    
	if ((sign = n) < 0)
		n = -n;

	do {
		s[i++] = digits[n % b];
	} while ((n /= b) > 0);

	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';

	return strrev(s);
}

/** *************************************************************
	@brief	Compares two strings
	@param 	in1 : first string
	@param 	in2 : second string
	@return	TRUE or FALSE
*****************************************************************/
uint8_t cmpstr(char *in1, char *in2)
{
  char *s1, *s2;
  int ret=TRUE;
  
  s1=in1;
  s2=in2;
  
  for(int x=0;x<=strlenght(s1);x++)
  {
    if(s1[x]!=s2[x])
    {
      ret=FALSE;
      break;
    }
  }
  
  return ret;
}

