#include <avr/io.h>  // RAMEND
#include "include/mem_check.h"

// Mask to init SRAM and check against
#define MASK 0xaa

// From linker script
extern unsigned char __heap_start;

// !!! This doesn't work together with malloc et.al. (whose use is
// !!! discouraged on AVR, anyway). alloca, however, is no problem
// !!! because it allocates on stack.

/** *************************************************************
	@brief	Get minimum of free ram from now up
	@param 	none
	@return	value in bytes
*****************************************************************/
unsigned short GET_UNUSED_RAM (void)
{
   unsigned short unused = 0;
   unsigned char *p = &__heap_start;

   do
   {
      if (*p++ != MASK)
         break;

      unused++;
   } while (p <= (unsigned char*) RAMEND);

   return unused;
}
unsigned int GET_FREE_RAM (void)
{
	return SP - (uint16_t) &__heap_start;
}

/** *************************************************************
	@brief	NEVER CALL THIS FUNCTION
	@brief	IT'S PART OF INIT SYSTEM
*****************************************************************/
void __attribute__ ((naked, section(".init3"))) init_mem (void);
void init_mem (void)
{
   //  Use inline assembler so it works even with optimization turned off
   __asm volatile (
      "ldi r30, lo8 (__heap_start)"  "\n\t"
      "ldi r31, hi8 (__heap_start)"  "\n\t"
      "ldi r24, %0"                  "\n\t"
      "ldi r25, hi8 (%1)"            "\n"
      "0:"                           "\n\t"
      "st  Z+,  r24"                 "\n\t"
      "cpi r30, lo8 (%1)"            "\n\t"
      "cpc r31, r25"                 "\n\t"
      "brlo 0b"
         :
         : "i" (MASK), "i" (RAMEND+1)
   );
}
// From linker script
extern unsigned char __heap_start;


